def merge_sort(arr):
    if len(arr) <= 1:
        return arr

    mid = len(arr) // 2
    left_half = arr[:mid]
    right_half = arr[mid:]

    # Recursively sort the left and right halves
    left_half = merge_sort(left_half)
    right_half = merge_sort(right_half)

    # Merge the sorted halves
    return merge(left_half, right_half)

def merge(left, right):
    merged = []
    left_index = 0
    right_index = 0

    # Merge smaller elements first
    while left_index < len(left) and right_index < len(right):
        if left[left_index] <= right[right_index]:
            merged.append(left[left_index])
            left_index += 1
        else:
            merged.append(right[right_index])
            right_index += 1

    # Append remaining elements from left or right halves
    merged.extend(left[left_index:])
    merged.extend(right[right_index:])

    return merged



def insertion_sort(array):
    for i in range(1, len(array)):
        key = array[i]
        j = i - 1
        while j >= 0 and key < array[j]:
            array[j + 1] = array[j]
            j -= 1
        array[j + 1] = key
    return array


def selection_sort(arr):
    n = len(arr)
    for i in range(n):
        min_idx = i
        for j in range(i + 1, n):
            if arr[j] < arr[min_idx]:
                min_idx = j
        arr[i], arr[min_idx] = arr[min_idx], arr[i]
    return arr



def quick_sort(arr):
    if len(arr) <= 1:
        return arr
    else:
        pivot = arr[0]
        less_than_pivot = [element for element in arr[1:] if element <= pivot]
        greater_than_pivot = [element for element in arr[1:] if element > pivot]
        return quick_sort(less_than_pivot) + [pivot] + quick_sort(greater_than_pivot)



from .bubble_sort import bubble_sort
from .counting_sort import counting_sort
from .insertion_sort import insertion_sort
from .merge_sort import merge_sort
from .quick_sort import quick_sort
from .radix_sort import radix_sort
from .selection_sort import selection_sort



def counting_sort(arr, exp):
    n = len(arr)
    output = [0] * n
    count = [0] * 10

    for i in range(n):
        index = arr[i] // exp
        count[index % 10] += 1

    for i in range(1, 10):
        count[i] += count[i - 1]

    i = n - 1
    while i >= 0:
        index = arr[i] // exp
        output[count[index % 10] - 1] = arr[i]
        count[index % 10] -= 1
        i -= 1

    for i in range(n):
        arr[i] = output[i]

def radix_sort(arr):
    max_val = max(arr)
    exp = 1
    while max_val // exp > 0:
        counting_sort(arr, exp)
        exp *= 10
    return arr



def counting_sort(array):
    if len(array) == 0:
        return array

    # Find the maximum value in the array
    max_val = max(array)
    size = len(array)

    # Initialize count array
    count = [0] * (max_val + 1)

    # Store the count of each element in the count array
    for num in array:
        count[num] += 1

    # Modify the count array to store the actual position of the element
    for i in range(1, max_val + 1):
        count[i] += count[i - 1]

    # Create the output array
    output = [0] * size

    # Build the output array
    i = size - 1
    while i >= 0:
        output[count[array[i]] - 1] = array[i]
        count[array[i]] -= 1
        i -= 1

    # Copy the sorted elements into the original array
    for i in range(size):
        array[i] = output[i]

    return array



def bubble_sort(arr):
    n = len(arr)
    for i in range(n):
        for j in range(0, n - i - 1):
            if arr[j] > arr[j + 1]:
                arr[j], arr[j + 1] = arr[j + 1], arr[j]
    return arr



from setuptools import setup, find_packages

setup(
    name="sortByTanvir",
    version="0.0.2",
    packages=find_packages(),
    install_requires=[],
    entry_points={},
    description="A simple sorting algorithms education package",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Tanvir-yzu",
    author_email="2020tanvir1971@gmail.com",
    url="https://github.com/Tanvir-yzu/sortByTanvir.git",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
    ],
    keywords=["sorting algorithms education","Tanvir-yzu","sortByTanvir","sorting"]
)

